from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

logo = SchLib(tool=SKIDL).add_parts(*[
        Part(name='OPEN_HARDWARE_1',dest=TEMPLATE,tool=SKIDL,keywords='Logo',description='Open Hardware Logo',ref_prefix='LOGO',num_units=1,do_erc=True),
        Part(name='OPEN_HARDWARE_2',dest=TEMPLATE,tool=SKIDL,keywords='Logo',description='Large Open hardware Logo',ref_prefix='LOGO',num_units=1,do_erc=True)])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

switches = SchLib(tool=SKIDL).add_parts(*[
        Part(name='SW_Coded',dest=TEMPLATE,tool=SKIDL,keywords='rotary hex',description='Rotary switch, 4-bit encoding',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='CM',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='D0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='D1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='D2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='D3',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_DIP_x01',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='1x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x1*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True)]),
        Part(name='SW_DIP_x02',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='2x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x2*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True)]),
        Part(name='SW_DIP_x03',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='3x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x3*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True)]),
        Part(name='SW_DIP_x04',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='4x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x4*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True)]),
        Part(name='SW_DIP_x05',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='5x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x5*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True)]),
        Part(name='SW_DIP_x06',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='6x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x6*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True)]),
        Part(name='SW_DIP_x07',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='7x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x7*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True)]),
        Part(name='SW_DIP_x08',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='8x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x8*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',do_erc=True),
            Pin(num='16',name='~',do_erc=True)]),
        Part(name='SW_DIP_x09',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='9x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x9*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',do_erc=True),
            Pin(num='16',name='~',do_erc=True),
            Pin(num='17',name='~',do_erc=True),
            Pin(num='18',name='~',do_erc=True)]),
        Part(name='SW_DIP_x10',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='10x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x10*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='20',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',do_erc=True),
            Pin(num='16',name='~',do_erc=True),
            Pin(num='17',name='~',do_erc=True),
            Pin(num='18',name='~',do_erc=True),
            Pin(num='19',name='~',do_erc=True)]),
        Part(name='SW_DIP_x11',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='11x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x11*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='20',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='21',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='22',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='15',name='~',do_erc=True),
            Pin(num='16',name='~',do_erc=True),
            Pin(num='17',name='~',do_erc=True),
            Pin(num='18',name='~',do_erc=True),
            Pin(num='19',name='~',do_erc=True)]),
        Part(name='SW_DIP_x12',dest=TEMPLATE,tool=SKIDL,keywords='dip switch',description='12x DIP Switch, Single Pole Single Throw (SPST) switch, small symbol',ref_prefix='SW',num_units=1,fplist=['SW?DIP?x12*'],do_erc=True,pins=[
            Pin(num='1',name='~',do_erc=True),
            Pin(num='2',name='~',do_erc=True),
            Pin(num='3',name='~',do_erc=True),
            Pin(num='4',name='~',do_erc=True),
            Pin(num='5',name='~',do_erc=True),
            Pin(num='6',name='~',do_erc=True),
            Pin(num='7',name='~',do_erc=True),
            Pin(num='8',name='~',do_erc=True),
            Pin(num='9',name='~',do_erc=True),
            Pin(num='10',name='~',do_erc=True),
            Pin(num='20',name='~',do_erc=True),
            Pin(num='11',name='~',do_erc=True),
            Pin(num='21',name='~',do_erc=True),
            Pin(num='12',name='~',do_erc=True),
            Pin(num='22',name='~',do_erc=True),
            Pin(num='13',name='~',do_erc=True),
            Pin(num='23',name='~',do_erc=True),
            Pin(num='14',name='~',do_erc=True),
            Pin(num='24',name='~',do_erc=True),
            Pin(num='15',name='~',do_erc=True),
            Pin(num='16',name='~',do_erc=True),
            Pin(num='17',name='~',do_erc=True),
            Pin(num='18',name='~',do_erc=True),
            Pin(num='19',name='~',do_erc=True)]),
        Part(name='SW_DPDT_x2',dest=TEMPLATE,tool=SKIDL,keywords='switch dual-pole double-throw DPDT spdt ON-ON',description='Switch, dual pole double throw, separate symbols',ref_prefix='SW',num_units=2,do_erc=True,pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='B',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='B',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_DPST',dest=TEMPLATE,tool=SKIDL,keywords='switch dual double-pole single-throw OFF-ON',description='Double Pole Single Throw (DPST) Switch',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',do_erc=True),
            Pin(num='2',name='2',do_erc=True),
            Pin(num='3',name='3',do_erc=True),
            Pin(num='4',name='4',do_erc=True)]),
        Part(name='SW_DPST_Temperature',dest=TEMPLATE,tool=SKIDL,keywords='temerature switch dual double-pole single-throw OFF-ON',description='Double Pole Single Throw (DPST) Switch, temperature dependent',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',do_erc=True),
            Pin(num='2',name='2',do_erc=True),
            Pin(num='3',name='3',do_erc=True),
            Pin(num='4',name='4',do_erc=True)]),
        Part(name='SW_DPST_x2',dest=TEMPLATE,tool=SKIDL,keywords='switch lever',description='Single Pole Single Throw (SPST) switch, separate symbol',ref_prefix='SW',num_units=2,do_erc=True,pins=[
            Pin(num='1',name='A',do_erc=True),
            Pin(num='2',name='B',do_erc=True),
            Pin(num='3',name='A',do_erc=True),
            Pin(num='4',name='B',do_erc=True)]),
        Part(name='SW_E3_SA3216',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button LCD',description='Push button switch with LCD screen',ref_prefix='SW',num_units=1,do_erc=True,aliases=['SW_E3_SA3624', 'SW_E3_SA6432', 'SW_MMI_Q5-100'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='5V',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='CLK',do_erc=True),
            Pin(num='4',name='DAT',do_erc=True),
            Pin(num='5',name='SW1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='SW2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button',description='Push button switch, generic, two pins',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_45deg',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button',description='Push button switch, normally open, two pins, 45° tilted',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Dual',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button',description='Push button switch, generic, symbol, four pins',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Dual_x2',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button',description='Push button switch, generic, separate symbols, four pins',ref_prefix='SW',num_units=2,do_erc=True,pins=[
            Pin(num='3',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='B',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_LED',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button LED',description='Push button switch with LED, generic',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Lamp',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-open pushbutton push-button Lamp',description='Push button switch with Signal Lamp, generic',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='L',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='L',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Open',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-closed pushbutton push-button',description='Push button switch, push-to-open, generic, two pins',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='B',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Open_Dual',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-closed pushbutton push-button',description='Push button switch, normally closed, generic, four pins',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_Open_Dual_x2',dest=TEMPLATE,tool=SKIDL,keywords='switch normally-closed pushbutton push-button',description='Push button switch, push-to-open, generic, two pins',ref_prefix='SW',num_units=2,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='2',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Push_SPDT',dest=TEMPLATE,tool=SKIDL,keywords='switch single-pole double-throw spdt ON-ON',description='Momentary Switch, single pole double throw',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='B',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Reed',dest=TEMPLATE,tool=SKIDL,keywords='reed magnetic switch',description='reed switch',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',do_erc=True),
            Pin(num='2',name='2',do_erc=True)]),
        Part(name='SW_Reed_Opener',dest=TEMPLATE,tool=SKIDL,keywords='reed magnetic switch',description='reed switch, default-closed',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',do_erc=True),
            Pin(num='2',name='2',do_erc=True)]),
        Part(name='SW_Reed_SPDT',dest=TEMPLATE,tool=SKIDL,keywords='reed magnetic switch SPDT',description='SPDT reed switch',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',do_erc=True),
            Pin(num='2',name='2',do_erc=True),
            Pin(num='3',name='3',do_erc=True)]),
        Part(name='SW_Rotary12',dest=TEMPLATE,tool=SKIDL,keywords='rotary switch',description='rotary switch with 12 positions',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='5',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='6',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='8',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='9',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='10',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='11',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='12',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='13',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Rotary2x6',dest=TEMPLATE,tool=SKIDL,keywords='rotary switch',description='2 rotary switch with 6 positions',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='5',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='6',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='8',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='9',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='10',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='11',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='12',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='13',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='14',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Rotary3x4',dest=TEMPLATE,tool=SKIDL,keywords='rotary switch',description='3 rotary switches with 4 positions',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='5',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='6',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='8',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='9',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='10',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='11',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='12',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='13',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='14',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='15',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_Rotary4x3',dest=TEMPLATE,tool=SKIDL,keywords='rotary switch',description='4 rotary switches with 3 positions',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='5',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='6',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='7',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='8',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='9',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='10',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='11',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='12',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='13',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='14',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='15',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='16',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SP3T',dest=TEMPLATE,tool=SKIDL,keywords='switch sp3t ON-ON-ON',description='Switch, three position, single pole triple throw, 3 position switch, SP3T',ref_prefix='SW',num_units=1,fplist=['SW*', 'SP3T*'],do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='4',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SPDT',dest=TEMPLATE,tool=SKIDL,keywords='switch single-pole double-throw spdt ON-ON',description='Switch, single pole double throw',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='A',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='B',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='C',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SPDT_MSM',dest=TEMPLATE,tool=SKIDL,keywords='switch spdt single-pole double-throw ON-OFF-ON',description='Switch, single pole double throw, center OFF position',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='3',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SPST',dest=TEMPLATE,tool=SKIDL,keywords='switch lever',description='Single Pole Single Throw (SPST) switch',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='A',do_erc=True),
            Pin(num='2',name='B',do_erc=True)]),
        Part(name='SW_SPST_LED',dest=TEMPLATE,tool=SKIDL,keywords='switch SPST LED OFF-ON',description='Single Pole Single Throw (SPST) switch with LED, generic',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='K',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='A',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SPST_Lamp',dest=TEMPLATE,tool=SKIDL,keywords='switch SPST LED OFF-ON lamp',description='Single Pole Single Throw (SPST) switch with signal lamp, generic',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='L',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='L',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='SW_SPST_Temperature',dest=TEMPLATE,tool=SKIDL,keywords='temperature switch',description='Single Pole Single Throw (SPST) switch, temperature dependent',ref_prefix='SW',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='A',do_erc=True),
            Pin(num='2',name='B',do_erc=True)])])


# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handles schematic libraries for various ECAD tools.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import str
from future import standard_library
standard_library.install_aliases()

from .utilities import *


class SchLib(object):
    """
    A class for storing parts from a schematic component library file.

    Attributes:
        filename: The name of the file from which the parts were read.
        parts: The list of parts (composed of Part objects).

    Args:
        filename: The name of the library file.
        tool: The format of the library file (e.g., KICAD).

    Keyword Args:
        attribs: Key/value pairs of attributes to add to the library.
    """

    # Keep a dict of filenames and their associated SchLib object
    # for fast loading of libraries.
    _cache = {}

    def __init__(self, filename=None, tool=None, **attribs):
        """
        Load the parts from a library file.
        """

        import skidl

        if tool is None:
            tool = skidl.get_default_tool()

        # Library starts off empty of parts.
        self.parts = []

        # Attach attributes to the library.
        for k, v in attribs.items():
            setattr(self, k, v)

        # If no filename, create an empty library.
        if not filename:
            pass

        # Load this SchLib with an existing SchLib object if the file name
        # matches one in the cache.
        elif filename in self._cache:
            self.__dict__.update(self._cache[filename].__dict__)

        # Otherwise, load from a schematic library file.
        else:
            try:
                # Use the tool name to find the function for loading the library.
                load_func = getattr(self, '_load_sch_lib_{}'.format(tool))
            except AttributeError:
                # OK, that didn't work so well...
                logger.error('Unsupported ECAD tool library: {}.'.format(tool))
                raise Exception
            else:
                load_func(filename, skidl.lib_search_paths[tool])
                self.filename = filename
                # Cache a reference to the library.
                self._cache[filename] = self

    @classmethod
    def reset(cls):
        """Clear the cache of processed library files."""
        cls._cache = {}

    def add_parts(self, *parts):
        """Add one or more parts to a library."""

        from .defines import TEMPLATE

        for part in flatten(parts):
            # Parts with the same name are not allowed in the library.
            # Also, do not check the backup library to see if the parts
            # are in there because that's probably a different library.
            if not self.get_parts(
                    use_backup_lib=False, name=re.escape(part.name)):
                self.parts.append(part.copy(dest=TEMPLATE))
        return self

    __iadd__ = add_parts

    def get_parts(self, use_backup_lib=True, **criteria):
        """
        Return parts from a library that match *all* the given criteria.

        Keyword Args:
            criteria: One or more keyword-argument pairs. The keyword specifies
                the attribute name while the argument contains the desired value
                of the attribute.

        Returns:
            A single Part or a list of Parts that match all the criteria.
        """

        import skidl

        parts = list_or_scalar(filter_list(self.parts, **criteria))
        if not parts and use_backup_lib and skidl.QUERY_BACKUP_LIB:
            try:
                backup_lib_ = skidl.load_backup_lib()
                parts = backup_lib_.get_parts(use_backup_lib=False, **criteria)
            except AttributeError:
                pass
        return parts

    def get_part_by_name(self, name, allow_multiples=False, silent=False):
        """
        Return a Part with the given name or alias from the part list.

        Args:
            name: The part name or alias to search for in the library.
            allow_multiples: If true, return a list of parts matching the name.
                If false, return only the first matching part and issue
                a warning if there were more than one.
            silent: If true, don't issue errors or warnings.

        Returns:
            A single Part or a list of Parts that match all the criteria.
        """

        # First check to see if there is a part or parts with a matching name.
        parts = self.get_parts(name=name)

        # No part with that name, so check for an alias that matches.
        if not parts:
            parts = self.get_parts(aliases=name)

            # No part with that alias either, so signal an error.
            if not parts:
                if not silent:
                    logger.error(
                        'Unable to find part {} in library {}.'.format(
                            name, getattr(self, 'filename', 'UNKNOWN')))
                raise Exception

        # Multiple parts with that name or alias exists, so return the list
        # of parts or just the first part on the list.
        if isinstance(parts, (list, tuple)):

            # Return the entire list if multiples are allowed.
            if allow_multiples:
                parts = [p.parse() for p in parts]

            # Just return the first part from the list if multiples are not
            # allowed and issue a warning.
            else:
                if not silent:
                    logger.warning(
                        'Found multiple parts matching {}. Selecting {}.'.
                        format(name, parts[0].name))
                parts = parts[0]
                parts.parse()

        # Only a single matching part was found, so return that.
        else:
            parts.parse()

        # Return the library part or parts that were found.
        return parts

    # Get part by name or alias using []'s.
    __getitem__ = get_part_by_name

    def __str__(self):
        """Return a list of the part names in this library as a string."""
        return '\n'.join(
            ['{}: {}'.format(p.name, p.description) for p in self.parts])

    __repr__ = __str__

    def export(self, libname, file_=None, tool=None):
        """
        Export a library into a file.

        Args:
            libname: A string containing the name of the library.
            file_: The file the library will be exported to. It can either
                be a file object or a string or None. If None, the file
                will be the same as the library name with the library
                suffix appended.
            tool: The CAD tool library format to be used. Currently, this can
                only be SKIDL.
        """

        def prettify(s):
            """Breakup and indent library export string."""
            s = re.sub(r'(Part\()', r'\n        \1', s)
            s = re.sub(r'(Pin\()', r'\n            \1', s)
            return s

        import skidl
        from .defines import SKIDL

        if tool is None:
            tool = SKIDL

        if not file_:
            file_ = libname + skidl.lib_suffixes[tool]

        export_str = 'from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE\n\n'
        export_str += "SKIDL_lib_version = '0.0.1'\n\n"
        part_export_str = ','.join([p.export() for p in self.parts])
        export_str += '{} = SchLib(tool=SKIDL).add_parts(*[{}])'.format(
            cnvt_to_var_name(libname), part_export_str)
        export_str = prettify(export_str)
        with opened(file_, "w") as f:
            f.write(export_str)

    def __len__(self):
        """
        Return number of parts in library.
        """
        return len(self.parts)



# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Functions for finding/displaying parts.
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from future import standard_library
standard_library.install_aliases()
import os
import os.path

from .py_2_3 import *  # pylint: disable=wildcard-import
from .utilities import *


def search(term, tool=None):
    """
    Print a list of components with the regex term within their name, alias, description or keywords.
    """

    import skidl
    from .SchLib import SchLib

    def search_libraries(term, tool):
        """Search for a regex term in part libraries."""

        parts = set(
        )  # Set of parts and their containing libraries found with the term.

        for lib_dir in skidl.lib_search_paths[tool]:
            print('lib_dir = {}'.format(lib_dir))
            # Get all the library files in the search path.
            try:
                lib_files = os.listdir(lib_dir)
            except (FileNotFoundError, OSError):
                logger.warning("Could not open directory '{}'".format(lib_dir))
                lib_files = list(
                )  # Empty list since library directory was not found.
            lib_files = [
                l for l in lib_files if l.endswith(lib_suffixes[tool])
            ]

            for lib_file in lib_files:
                print(
                    ' ' * 79, '\rSearching {} ...'.format(lib_file), end='\r')
                lib = SchLib(
                    os.path.join(lib_dir, lib_file),
                    tool=tool)  # Open the library file.

                def mk_list(l):
                    """Make a list out of whatever is given."""
                    if isinstance(l, (list, tuple)):
                        return l
                    if not l:
                        return []
                    return [l]

                # Search the current library for parts with the given term in
                # each of the these categories.
                for category in ['name', 'alias', 'description', 'keywords']:
                    for part in mk_list(lib.get_parts(**{category: term})):
                        part.parse(
                        )  # Parse the part to instantiate the complete object.
                        parts.add(
                            (lib_file,
                             part))  # Store the library name and part object.
                print(' ' * 79, end='\r')

        return list(
            parts)  # Return the list of parts and their containing libraries.

    if tool is None:
        tool = skidl.get_default_tool()

    term = '.*' + term + '.*'  # Use the given term as a substring.
    parts = search_libraries(term,
                             tool)  # Search for parts with that substring.

    # Print each part name sorted by the library where it was found.
    for lib_file, p in sorted(parts, key=lambda p: p[0]):
        print('{}: {} ({})'.format(lib_file, getattr(p, 'name', '???'),
                                   getattr(p, 'description', '???')))


def show(lib, part_name, tool=None):
    """
    Print the I/O pins for a given part in a library.

    Args:
        lib: Either a SchLib object or the name of a library.
        part_name: The name of the part in the library.
        tool: The ECAD tool format for the library.

    Returns:
        A Part object.
    """

    import skidl
    from .Part import Part
    from .defines import TEMPLATE

    if tool is None:
        tool = skidl.get_default_tool()
    try:
        return Part(
            lib, re.escape(part_name), tool=tool, dest=TEMPLATE)
    except Exception:
        return None



import os
from skidl import *

this_file_dir = os.path.dirname(os.path.abspath(__file__))

files_at_start = set([])

def setup_function(f):
    # Record files originally in directory so we know which ones not to delete.
    global files_at_start
    files_at_start = set(os.listdir(os.getcwd()))

    default_circuit.reset()

    lib_search_paths.clear()
    lib_search_paths.update({
        KICAD: [os.getcwd(), this_file_dir],
        SKIDL: [os.getcwd(), this_file_dir, get_filename("../skidl/libs")],
        SPICE: [os.getcwd(), this_file_dir]
    })

    set_default_tool(INITIAL_DEFAULT_TOOL)
    set_query_backup_lib(INITIAL_QUERY_BACKUP_LIB)

def teardown_function(f):
    # Delete files created during testing.
    files_at_end = set(os.listdir(os.getcwd()))
    for file in files_at_end - files_at_start:
        try:
            os.remove(file)
        except Exception:
            pass

def get_filename(fn):
    """
    Resolves a filename relative to the "tests" directory.
    """
    abs_fn = \
        fn if os.path.isabs(fn) else \
        os.path.join(this_file_dir, fn)
    return os.path.realpath(abs_fn)

if __name__ == '__main__':
    setup_function(None)
    with open('test.txt','wb') as f:
        f.write('test')
    teardown_function(None)



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

relays = SchLib(tool=SKIDL).add_parts(*[
        Part(name='AZ850-x',dest=TEMPLATE,tool=SKIDL,keywords='Miniature Polarised Relay Dual Pole Bistable',description='Microminiature Polarised Dual Pole Relay Bistable',ref_prefix='RL',num_units=1,do_erc=True,aliases=['AZ850P1-x'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='AZ850P2-x',dest=TEMPLATE,tool=SKIDL,keywords='Miniature Polarised Relay Dual Pole Bistable',description='Microminiature Polarised Dual Pole Relay Bistable',ref_prefix='RL',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FINDER-32.21-x300',dest=TEMPLATE,tool=SKIDL,keywords='Single Pole Relay SPST-NO',description='RELPOL RM50-xx21, Single Pole Relay SPST-NO, 6A',ref_prefix='RL',num_units=1,do_erc=True,aliases=['FINDER-36.11-4301', 'RM50-xx21'],pins=[
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FINDER-40.51',dest=TEMPLATE,tool=SKIDL,keywords='Single Pole Relay Sugar Cube',description='Single Pole Relay, Sugar Cube, 15A',ref_prefix='RL',num_units=1,do_erc=True,aliases=['FINDER-36.11.4001', 'FINDER-40.31', 'RM50-xx11'],pins=[
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FINDER-40.52',dest=TEMPLATE,tool=SKIDL,keywords='Dual Pole Relay',description='RELPOL RSM822, Dual Pole Relay, 5mm Pitch, 2A',ref_prefix='RL',num_units=1,do_erc=True,aliases=['FINDER-41.52', 'FINDER-44.52', 'FINDER-44.62', 'FINDER-30.22', 'RSM822', 'RM84'],pins=[
            Pin(num='11',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='22',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='A2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='24',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FRT5',dest=TEMPLATE,tool=SKIDL,keywords='relay monostable',description='FRT5 relay',ref_prefix='K',num_units=1,fplist=['Relay_DPDT_FRT5'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='FRT5_separated',dest=TEMPLATE,tool=SKIDL,keywords='relay monostable',description='FRT5 relay',ref_prefix='K',num_units=3,fplist=['Relay_DPDT_FRT5'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='G5Q-1',dest=TEMPLATE,tool=SKIDL,keywords='Miniature Single Pole Relay',description='OMRON G5G-1, Miniature Single Pole Relay, SPDT, 10A',ref_prefix='RL',num_units=1,fplist=['Relay?SPDT?OMRON?G5Q*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='G5Q-1A',dest=TEMPLATE,tool=SKIDL,keywords='Miniature Single Pole Relay',description='G5G-1A, Miniature Single Pole Relay, SPST-NO, 10A',ref_prefix='RL',num_units=1,fplist=['Relay?SPST?NO?OMRON?G5Q*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IM00',dest=TEMPLATE,tool=SKIDL,keywords='relay monostable',description='IM Relay, standard version, monostable, switching current 2/5A, power 60W/62.5VA, voltage 220VDC/250VAC',ref_prefix='K',num_units=1,do_erc=True,aliases=['IM01', 'IM02', 'IM03', 'IM04', 'IM05', 'IM06', 'IM07', 'IM08'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IM11',dest=TEMPLATE,tool=SKIDL,keywords='relay monastable sensitive',description='IM Relay, sensitive version, monostable, switching current 2/5A, power 60W/62.5VA, voltage 220VDC/250VAC',ref_prefix='K',num_units=1,do_erc=True,aliases=['IM12', 'IM13', 'IM16', 'IM17'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IM21',dest=TEMPLATE,tool=SKIDL,keywords='relay monastable sensitive',description='IM Relay, high sensitive version, monostable, switching current 2/5A, power 60W/62.5VA, voltage 220VDC/250VAC',ref_prefix='K',num_units=1,do_erc=True,aliases=['IM22', 'IM23', 'IM26'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='IM40',dest=TEMPLATE,tool=SKIDL,keywords='relay bistable',description='IM Relay, standard version, bistable, switching current 2/5A, power 60W/62.5VA, voltage 220VDC/250VAC',ref_prefix='K',num_units=1,do_erc=True,aliases=['IM41', 'IM42', 'IM43', 'IM44', 'IM45', 'IM46', 'IM47', 'IM48'],pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TIANBO-HJR-4102-L',dest=TEMPLATE,tool=SKIDL,keywords='Single Pole Relay',description='TIANBO HJR-4102-L, Single Pole Relay, 5mm Pitch, 3A',ref_prefix='RL',num_units=1,fplist=['Relay*HJR?4102*'],do_erc=True,pins=[
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~',func=Pin.PASSIVE,do_erc=True)])])


# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handler for reading Kicad libraries and generating netlists.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import str
from builtins import int
from builtins import range
from builtins import dict
from builtins import zip
from future import standard_library
standard_library.install_aliases()

import os.path
import time
from ..py_2_3 import *
from ..defines import *
from ..utilities import *
from ..pckg_info import __version__

tool_name = KICAD
lib_suffix = '.lib'


def _load_sch_lib_(self, filename=None, lib_search_paths_=None):
    """
    Load the parts from a KiCad schematic library file.

    Args:
        filename: The name of the KiCad schematic library file.
    """

    from ..skidl import lib_suffixes
    from ..Part import Part

    # Try to open the file. Add a .lib extension if needed. If the file
    # doesn't open, then try looking in the KiCad library directory.
    try:
        f, _ = find_and_open_file(filename, lib_search_paths_, lib_suffixes[KICAD])
    except Exception as e:
        raise Exception(
            'Unable to open KiCad Schematic Library File {} ({})'.format(
                filename, str(e)))

    # Check the file header to make sure it's a KiCad library.
    header = []
    header = [f.readline()]
    if header and 'EESchema-LIBRARY' not in header[0]:
        raise Exception(
            'The file {} is not a KiCad Schematic Library File.\n'.format(
                filename))

    # Read the definition of each part line-by-line and then create
    # a Part object that gets stored in the part list.
    part_defn = []
    for line in f.readlines():

        # Skip over comments.
        if line.startswith('#'):
            pass

        # Look for the start of a part definition.
        elif line.startswith('DEF'):
            # Initialize the part definition with the first line.
            # This will also signal that succeeding lines should be added.
            part_defn = [line]

        # If gathering the part definition has begun, then continue adding lines.
        elif part_defn:
            part_defn.append(line)

            # If the current line ends this part definition, then create
            # the Part object and add it to the part list. Be sure to
            # indicate that the Part object is being added to a library
            # and not to a schematic netlist.
            if line.startswith('ENDDEF'):
                self.add_parts(
                    Part(part_defn=part_defn, tool=KICAD, dest=LIBRARY))

                # Clear the part definition in preparation for the next one.
                part_defn = []

    # Now add information from any associated DCM file.
    filename = os.path.splitext(filename)[0]  # Strip any extension.
    f, _ = find_and_open_file(
        filename, lib_search_paths_, '.dcm', allow_failure=True)
    if not f:
        return

    part_desc = {}
    for line in f.readlines():

        # Skip over comments.
        if line.startswith('#'):
            pass

        # Look for the start of a part description.
        elif line.startswith('$CMP'):
            part_desc['name'] = line.split()[-1]

        # If gathering the part definition has begun, then continue adding lines.
        elif part_desc:
            if line.startswith('D'):
                part_desc['description'] = ' '.join(line.split()[1:])
            elif line.startswith('K'):
                part_desc['keywords'] = ' '.join(line.split()[1:])
            elif line.startswith('$ENDCMP'):
                try:
                    part = self.get_part_by_name(
                        part_desc['name'], silent=True)
                except Exception:
                    pass
                else:
                    part.description = part_desc.get('description', '')
                    part.keywords = part_desc.get('keywords', '')
                part_desc = {}
            else:
                pass

def _parse_lib_part_(self, just_get_name=False):
    """
    Create a Part using a part definition from a KiCad schematic library.

    This method was written based on the code from
    https://github.com/KiCad/kicad-library-utils/tree/master/schlib.
    It's covered by GPL3.

    Args:
        part_defn: A list of strings that define the part (usually read from a
            schematic library file). Can also be None.
        just_get_name: If true, scan the part definition until the
            name and aliases are found. The rest of the definition
            will be parsed if the part is actually used.
    """

    from ..Pin import Pin

    _DEF_KEYS = [
        'name', 'reference', 'unused', 'text_offset', 'draw_pinnumber',
        'draw_pinname', 'unit_count', 'units_locked', 'option_flag'
    ]
    _F0_KEYS = [
        'reference', 'posx', 'posy', 'text_size', 'text_orient',
        'visibility', 'htext_justify', 'vtext_justify'
    ]
    _FN_KEYS = [
        'name', 'posx', 'posy', 'text_size', 'text_orient', 'visibility',
        'htext_justify', 'vtext_justify', 'fieldname'
    ]
    _ARC_KEYS = [
        'posx', 'posy', 'radius', 'start_angle', 'end_angle', 'unit',
        'convert', 'thickness', 'fill', 'startx', 'starty', 'endx', 'endy'
    ]
    _CIRCLE_KEYS = [
        'posx', 'posy', 'radius', 'unit', 'convert', 'thickness', 'fill'
    ]
    _POLY_KEYS = [
        'point_count', 'unit', 'convert', 'thickness', 'points', 'fill'
    ]
    _RECT_KEYS = [
        'startx', 'starty', 'endx', 'endy', 'unit', 'convert', 'thickness',
        'fill'
    ]
    _TEXT_KEYS = [
        'direction', 'posx', 'posy', 'text_size', 'text_type', 'unit',
        'convert', 'text', 'italic', 'bold', 'hjustify', 'vjustify'
    ]
    _PIN_KEYS = [
        'name', 'num', 'posx', 'posy', 'length', 'direction',
        'name_text_size', 'num_text_size', 'unit', 'convert',
        'electrical_type', 'pin_type'
    ]
    _DRAW_KEYS = {
        'arcs': _ARC_KEYS,
        'circles': _CIRCLE_KEYS,
        'polylines': _POLY_KEYS,
        'rectangles': _RECT_KEYS,
        'texts': _TEXT_KEYS,
        'pins': _PIN_KEYS
    }
    _DRAW_ELEMS = {
        'arcs': 'A',
        'circles': 'C',
        'polylines': 'P',
        'rectangles': 'S',
        'texts': 'T',
        'pins': 'X'
    }
    _KEYS = {
        'DEF': _DEF_KEYS,
        'F0': _F0_KEYS,
        'F': _FN_KEYS,
        'A': _ARC_KEYS,
        'C': _CIRCLE_KEYS,
        'P': _POLY_KEYS,
        'S': _RECT_KEYS,
        'T': _TEXT_KEYS,
        'X': _PIN_KEYS
    }

    # Return if there's nothing to do (i.e., part has already been parsed).
    if not self.part_defn:
        return

    self.fplist = []  # Footprint list.
    self.aliases = []  # Part aliases.
    building_fplist = False  # True when working on footprint list in defn.
    building_draw = False  # True when gathering part drawing from defn.

    # Always make sure there are drawing & footprint sections, even if the part doesn't have them.
    self.draw = {
        'arcs': [],
        'circles': [],
        'polylines': [],
        'rectangles': [],
        'texts': [],
        'pins': []
    }
    self.fplist = []

    # Go through the part definition line-by-line.
    for line in self.part_defn:

        # Split the line into words.

        line = line.replace('\n', '')

        # Extract all the non-quoted and quoted text pieces, accounting for escaped quotes.
        unqu = r'[^\s"]+'  # Word without spaces or double-quotes.
        qu = r'(?<!\\)".*?(?<!\\)"'  # Quoted string, possibly with escaped quotes.
        srch = '|'.join([unqu + qu, qu, unqu])
        line = re.findall(
            srch, line)  # Replace line with list of pieces of line.

        # The first word indicates the type of part definition data that will follow.
        if line[0] in _KEYS:
            # Get the keywords for the current part definition data.
            key_list = _KEYS[line[0]]
            # Make a list of the values in the part data associated with each key.
            # Use an empty string for any missing values so every key will be
            # associated with something.
            values = line[1:] + [
                '' for _ in range(len(key_list) - len(line[1:]))
            ]

        # Create a dictionary of part definition keywords and values.
        if line[0] == 'DEF':
            self.definition = dict(list(zip(_DEF_KEYS, values)))
            self.name = self.definition['name']

            # To handle libraries quickly, just get the name and
            # aliases and only parse the rest of the part definition later.
            if just_get_name:
                if self.aliases:
                    # Name found, aliases already found so we're done.
                    return
                # Name found so scan defn to see if aliases are present.
                # (The majority of parts don't have aliases.)
                for ln in self.part_defn:
                    if re.match(r'^\s*ALIAS\s', ln):
                        # Break and keep parsing defn if aliases are present.
                        break
                else:
                    # No aliases found, so part name is all that's needed.
                    return

        # End the parsing of the part definition.
        elif line[0] == 'ENDDEF':
            break

        # Create a dictionary of F0 part field keywords and values.
        elif line[0] == 'F0':
            self.fields = []
            self.fields.append(dict(list(zip(_F0_KEYS, values))))

        # Create a dictionary of the other part field keywords and values.
        elif line[0][0] == 'F':
            # Make a list of field values with empty strings for missing fields.
            values = line[1:] + [
                '' for _ in range(len(_FN_KEYS) - len(line[1:]))
            ]
            self.fields.append(dict(list(zip(_FN_KEYS, values))))

        # Create a list of part aliases.
        elif line[0] == 'ALIAS':
            self.aliases = [alias for alias in line[1:]]
            if just_get_name and self.name:
                # Aliases found, name already found so we're done.
                return

        # Start the list of part footprints.
        elif line[0] == '$FPLIST':
            building_fplist = True

        # End the list of part footprints.
        elif line[0] == '$ENDFPLIST':
            building_fplist = False

        # Start gathering the drawing primitives for the part symbol.
        elif line[0] == 'DRAW':
            building_draw = True

        # End the gathering of drawing primitives.
        elif line[0] == 'ENDDRAW':
            building_draw = False

        # Every other line is either a footprint or a drawing primitive.
        else:
            # If the footprint list is being built, then add this line to it.
            if building_fplist:
                self.fplist.append(line[0])

            # Else if the drawing primitives are being gathered, process the
            # current line to see what type of primitive is in play.
            elif building_draw:

                # Gather arcs.
                if line[0] == 'A':
                    self.draw['arcs'].append(
                        dict(list(zip(_ARC_KEYS, values))))

                # Gather circles.
                elif line[0] == 'C':
                    self.draw['circles'].append(
                        dict(list(zip(_CIRCLE_KEYS, values))))

                # Gather polygons.
                elif line[0] == 'P':
                    n_points = int(line[1])
                    points = line[5:5 + (2 * n_points)]
                    values = line[1:5] + [points]
                    if len(line) > (5 + len(points)):
                        values += [line[-1]]
                    else:
                        values += ['']
                    self.draw['polylines'].append(
                        dict(list(zip(_POLY_KEYS, values))))

                # Gather rectangles.
                elif line[0] == 'S':
                    self.draw['rectangles'].append(
                        dict(list(zip(_RECT_KEYS, values))))

                # Gather text.
                elif line[0] == 'T':
                    self.draw['texts'].append(
                        dict(list(zip(_TEXT_KEYS, values))))

                # Gather the pin symbols. This is what we really want since
                # this defines the names, numbers and attributes of the
                # pins associated with the part.
                elif line[0] == 'X':
                    self.draw['pins'].append(
                        dict(list(zip(_PIN_KEYS, values))))

                # Found something unknown in the drawing section.
                else:
                    msg = 'Found something strange in {} symbol drawing: {}.'.format(
                        self.name, line)
                    logger.warning(msg)

            # Found something unknown outside the footprint list or drawing section.
            else:
                msg = 'Found something strange in {} symbol definition: {}.'.format(
                    self.name, line)
                logger.warning(msg)

    # Define some shortcuts to part information.
    self.num_units = int(
        self.definition['unit_count'])  # # of units within the part.
    self.name = self.definition['name']  # Part name (e.g., 'LM324').
    self.ref_prefix = self.definition[
        'reference']  # Part ref prefix (e.g., 'R').

    # Clear the part reference field directly. Don't use the setter function
    # since it will try to generate and assign a unique part reference if
    # passed a value of None.
    self._ref = None

    # Make a Pin object from the information in the KiCad pin data fields.
    def kicad_pin_to_pin(kicad_pin):
        p = Pin()  # Create a blank pin.

        # Replicate the KiCad pin fields as attributes in the Pin object.
        # Note that this update will not give the pins valid references
        # to the current part, but we'll fix that soon.
        p.__dict__.update(kicad_pin)

        pin_type_translation = {
            'I': Pin.INPUT,
            'O': Pin.OUTPUT,
            'B': Pin.BIDIR,
            'T': Pin.TRISTATE,
            'P': Pin.PASSIVE,
            'U': Pin.UNSPEC,
            'W': Pin.PWRIN,
            'w': Pin.PWROUT,
            'C': Pin.OPENCOLL,
            'E': Pin.OPENEMIT,
            'N': Pin.NOCONNECT
        }
        p.func = pin_type_translation[p.electrical_type]  # pylint: disable=no-member, attribute-defined-outside-init

        return p

    self.pins = [kicad_pin_to_pin(p) for p in self.draw['pins']]

    # Make sure all the pins have a valid reference to this part.
    self.associate_pins()

    # Part definition has been parsed, so clear it out. This prevents a
    # part from being parsed more than once.
    self.part_defn = None

def _gen_netlist_(self):
    scr_dict = scriptinfo()
    src_file = os.path.join(scr_dict['dir'], scr_dict['source'])  # pylint: disable=unused-variable
    date = time.strftime('%m/%d/%Y %I:%M %p')  # pylint: disable=unused-variable
    tool = 'SKiDL (' + __version__ + ')'  # pylint: disable=unused-variable
    template = '(export (version D)\n' + \
               '  (design\n' + \
               '    (source "{src_file}")\n' + \
               '    (date "{date}")\n' + \
               '    (tool "{tool}"))\n'
    netlist = template.format(**locals())
    netlist += "  (components"
    for p in sorted(self.parts, key=lambda p: str(p.ref)):
        netlist += '\n' + p.generate_netlist_component(KICAD)
    netlist += ")\n"
    netlist += "  (nets"
    for code, n in enumerate(
            sorted(self.get_nets(), key=lambda n: str(n.name))):
        n.code = code
        netlist += '\n' + n.generate_netlist_net(KICAD)
    netlist += ")\n)\n"
    return netlist

def _gen_netlist_comp_(self):
    ref = add_quotes(self.ref)

    try:
        value = self.value
        if not value:
            value = self.name
    except AttributeError:
        try:
            value = self.name
        except AttributeError:
            value = self.ref_prefix
    value = add_quotes(value)

    try:
        footprint = self.footprint
    except AttributeError:
        logger.error('No footprint for {part}/{ref}.'.format(
            part=self.name, ref=ref))
        footprint = 'No Footprint'
    footprint = add_quotes(footprint)

    lib = add_quotes(getattr(self, 'lib', 'NO_LIB'))  # pylint: disable=unused-variable
    name = add_quotes(self.name)  # pylint: disable=unused-variable

    fields = ''
    for fld_name in self._get_fields():
        fld_value = add_quotes(self.__dict__[fld_name])
        if fld_value:
            fld_name = add_quotes(fld_name)
            fields += '\n        (field (name {fld_name}) {fld_value})'.format(
                **locals())
    if fields:
        fields = '      (fields' + fields
        fields += ')\n'

    template = '    (comp (ref {ref})\n' + \
               '      (value {value})\n' + \
               '      (footprint {footprint})\n' + \
               '{fields}' + \
               '      (libsource (lib {lib}) (part {name})))'
    txt = template.format(**locals())
    return txt

def _gen_netlist_net_(self):
    code = add_quotes(self.code)  # pylint: disable=unused-variable
    name = add_quotes(self.name)  # pylint: disable=unused-variable
    txt = '    (net (code {code}) (name {name})'.format(**locals())
    for p in sorted(self.get_pins(), key=str):
        part_ref = add_quotes(p.part.ref)  # pylint: disable=unused-variable
        pin_num = add_quotes(p.num)  # pylint: disable=unused-variable
        txt += '\n      (node (ref {part_ref}) (pin {pin_num}))'.format(
            **locals())
    txt += ')'
    return txt

def _gen_xml_(self):
    scr_dict = scriptinfo()
    src_file = os.path.join(scr_dict['dir'], scr_dict['source'])  # pylint: disable=unused-variable
    date = time.strftime('%m/%d/%Y %I:%M %p')  # pylint: disable=unused-variable
    tool = 'SKiDL (' + __version__ + ')'  # pylint: disable=unused-variable
    template = '<?xml version="1.0" encoding="UTF-8"?>\n' + \
               '<export version="D">\n' + \
               '  <design>\n' + \
               '    <source>{src_file}</source>\n' + \
               '    <date>{date}</date>\n' + \
               '    <tool>{tool}</tool>\n' + \
               '  </design>\n'
    netlist = template.format(**locals())
    netlist += '  <components>'
    for p in self.parts:
        netlist += '\n' + p.generate_xml_component(KICAD)
    netlist += '\n  </components>\n'
    netlist += '  <nets>'
    for code, n in enumerate(self.get_nets()):
        n.code = code
        netlist += '\n' + n.generate_xml_net(KICAD)
    netlist += '\n  </nets>\n'
    netlist += '</export>\n'
    return netlist

def _gen_xml_comp_(self):
    ref = self.ref

    try:
        value = self.value
        if not value:
            value = self.name
    except AttributeError:
        try:
            value = self.name
        except AttributeError:
            value = self.ref_prefix

    try:
        footprint = self.footprint  # pylint: disable=unused-variable
    except AttributeError:
        logger.error('No footprint for {part}/{ref}.'.format(
            part=self.name, ref=ref))
        footprint = 'No Footprint'

    lib = add_quotes(getattr(self, 'lib', 'NO_LIB'))  # pylint: disable=unused-variable
    name = self.name  # pylint: disable=unused-variable

    fields = ''
    for fld_name in self._get_fields():
        fld_value = self.__dict__[fld_name]
        if fld_value:
            fields += '\n        <field name="{fld_name}">{fld_value}</field>'.format(
                **locals())
    if fields:
        fields = '      <fields>' + fields
        fields += '\n      </fields>\n'

    template = '    <comp ref="{ref}">\n' + \
               '      <value>{value}</value>\n' + \
               '      <footprint>{footprint}</footprint>\n' + \
               '{fields}' + \
               '      <libsource lib="{lib}" part="{name}"/>\n' + \
               '    </comp>'
    txt = template.format(**locals())
    return txt

def _gen_xml_net_(self):
    code = self.code  # pylint: disable=unused-variable
    name = self.name  # pylint: disable=unused-variable
    txt = '    <net code="{code}" name="{name}">'.format(**locals())
    for p in self.get_pins():
        part_ref = p.part.ref  # pylint: disable=unused-variable
        pin_num = p.num  # pylint: disable=unused-variable
        txt += '\n      <node ref="{part_ref}" pin="{pin_num}"/>'.format(
            **locals())
    txt += '\n    </net>'
    return txt



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

Lattice = SchLib(tool=SKIDL).add_parts(*[
        Part(name='GAL16V8',dest=TEMPLATE,tool=SKIDL,keywords='GAL PLD 16V8',description='Programmable Logic Array, DIP-20/SOIC-20/PLCC-20',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*', 'SOIC*', 'SO*', 'PLCC*'],do_erc=True,pins=[
            Pin(num='10',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='I1/CLK',do_erc=True),
            Pin(num='2',name='I2',do_erc=True),
            Pin(num='3',name='I3',do_erc=True),
            Pin(num='4',name='I4',do_erc=True),
            Pin(num='5',name='I5',do_erc=True),
            Pin(num='6',name='I6',do_erc=True),
            Pin(num='7',name='I7',do_erc=True),
            Pin(num='8',name='I8',do_erc=True),
            Pin(num='9',name='I9',do_erc=True),
            Pin(num='11',name='I10/~OE~',do_erc=True),
            Pin(num='12',name='IO8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='13',name='IO7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='14',name='IO6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='15',name='IO5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='16',name='IO4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='17',name='I03',func=Pin.TRISTATE,do_erc=True),
            Pin(num='18',name='IO2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='19',name='IO1',func=Pin.TRISTATE,do_erc=True)]),
        Part(name='PAL16L8',dest=TEMPLATE,tool=SKIDL,keywords='PAL PLD 16L8',description='Programmable Logic Array, DIP-20',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*'],do_erc=True,pins=[
            Pin(num='10',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='I1',do_erc=True),
            Pin(num='2',name='I2',do_erc=True),
            Pin(num='3',name='I3',do_erc=True),
            Pin(num='4',name='I4',do_erc=True),
            Pin(num='5',name='I5',do_erc=True),
            Pin(num='6',name='I6',do_erc=True),
            Pin(num='7',name='I7',do_erc=True),
            Pin(num='8',name='I8',do_erc=True),
            Pin(num='9',name='I9',do_erc=True),
            Pin(num='11',name='I10',do_erc=True),
            Pin(num='12',name='IO8',func=Pin.TRISTATE,do_erc=True),
            Pin(num='13',name='IO7',func=Pin.TRISTATE,do_erc=True),
            Pin(num='14',name='IO6',func=Pin.TRISTATE,do_erc=True),
            Pin(num='15',name='IO5',func=Pin.TRISTATE,do_erc=True),
            Pin(num='16',name='IO4',func=Pin.TRISTATE,do_erc=True),
            Pin(num='17',name='I03',func=Pin.TRISTATE,do_erc=True),
            Pin(num='18',name='IO2',func=Pin.TRISTATE,do_erc=True),
            Pin(num='19',name='IO1',func=Pin.TRISTATE,do_erc=True)]),
        Part(name='PAL20L8',dest=TEMPLATE,tool=SKIDL,keywords='PAL PLD 20L8',description='Programmable Logic Array, DIP-24',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*'],do_erc=True,pins=[
            Pin(num='12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='1',name='I1',do_erc=True),
            Pin(num='2',name='I2',do_erc=True),
            Pin(num='3',name='I3',do_erc=True),
            Pin(num='4',name='I4',do_erc=True),
            Pin(num='5',name='I5',do_erc=True),
            Pin(num='6',name='I6',do_erc=True),
            Pin(num='7',name='I7',do_erc=True),
            Pin(num='8',name='I8',do_erc=True),
            Pin(num='9',name='I9',do_erc=True),
            Pin(num='10',name='I10',do_erc=True),
            Pin(num='20',name='I020',func=Pin.TRISTATE,do_erc=True),
            Pin(num='11',name='I11',do_erc=True),
            Pin(num='21',name='IO21',func=Pin.TRISTATE,do_erc=True),
            Pin(num='22',name='IO22',func=Pin.TRISTATE,do_erc=True),
            Pin(num='13',name='I13',do_erc=True),
            Pin(num='23',name='I23',do_erc=True),
            Pin(num='14',name='I14',do_erc=True),
            Pin(num='15',name='IO15',func=Pin.TRISTATE,do_erc=True),
            Pin(num='16',name='IO16',func=Pin.TRISTATE,do_erc=True),
            Pin(num='17',name='IO17',func=Pin.TRISTATE,do_erc=True),
            Pin(num='18',name='IO18',func=Pin.TRISTATE,do_erc=True),
            Pin(num='19',name='IO19',func=Pin.TRISTATE,do_erc=True)]),
        Part(name='PAL20RS10',dest=TEMPLATE,tool=SKIDL,keywords='PAL PLD 20RS10',description='Programmable Logic Array, DIP-24 (Narrow)',ref_prefix='U',num_units=1,fplist=['DIP*', 'PDIP*'],do_erc=True,pins=[
            Pin(num='1',name='CLK',do_erc=True),
            Pin(num='2',name='I0',do_erc=True),
            Pin(num='3',name='I1',do_erc=True),
            Pin(num='4',name='I2',do_erc=True),
            Pin(num='5',name='I3',do_erc=True),
            Pin(num='6',name='I4',do_erc=True),
            Pin(num='7',name='I5',do_erc=True),
            Pin(num='8',name='I6',do_erc=True),
            Pin(num='9',name='I7',do_erc=True),
            Pin(num='10',name='I8',do_erc=True),
            Pin(num='20',name='O3',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='I9',do_erc=True),
            Pin(num='21',name='O2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='O1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='~OE~',do_erc=True),
            Pin(num='23',name='O0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='O9',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='O8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='O7',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='O6',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='O5',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='O4',func=Pin.OUTPUT,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

Xicor = SchLib(tool=SKIDL).add_parts(*[
        Part(name='X9118',dest=TEMPLATE,tool=SKIDL,keywords='digital potentiometer',description='100k Digital Potentiometer, 1024 steps, TSSOP-14',ref_prefix='U',num_units=1,fplist=['TSSOP*4.4x5mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='1',name='V+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='A0',do_erc=True),
            Pin(num='4',name='SCL',do_erc=True),
            Pin(num='6',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='V-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='A1',do_erc=True),
            Pin(num='14',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='~WP',do_erc=True),
            Pin(num='11',name='RW0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='RH0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='RL0',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='X9250',dest=TEMPLATE,tool=SKIDL,description='Quad 100k Digital Potentiometer, 256 steps, TSSOP-24/SOIC-24',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x15.4mm*Pitch1.27mm*', 'TSSOP*4.4x7.8mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='2',name='A0',do_erc=True),
            Pin(num='6',name='V+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~CS',do_erc=True),
            Pin(num='13',name='SIN',do_erc=True),
            Pin(num='23',name='SCL',do_erc=True),
            Pin(num='14',name='A1',do_erc=True),
            Pin(num='24',name='~HOLD',do_erc=True),
            Pin(num='18',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='V-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='1',name='SOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='RW3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='RH3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='RL3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='RL0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='RH0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='RW0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='RW2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='RH2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~WP',do_erc=True),
            Pin(num='22',name='RL2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='RL1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='RH1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='RW1',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='X9258',dest=TEMPLATE,tool=SKIDL,description='Quad 50k Digital Potentiometer, 256 steps, TSSOP-24/SOIC-24',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x15.4mm*Pitch1.27mm*', 'TSSOP*4.4x7.8mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='2',name='A0',do_erc=True),
            Pin(num='6',name='V+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='A2',do_erc=True),
            Pin(num='13',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='SCL',do_erc=True),
            Pin(num='14',name='A1',do_erc=True),
            Pin(num='24',name='A3',do_erc=True),
            Pin(num='18',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='V-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='RW3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='RH3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='RL3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='RL0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='RH0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='RW0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='RW2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='RH2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='~WP',do_erc=True),
            Pin(num='22',name='RL2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='RL1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='RH1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='RW1',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='X9258-PART',dest=TEMPLATE,tool=SKIDL,description='X9258, Quad 50k Digital Potentiometer, 256 steps, TSSOP-24/SOIC-24',ref_prefix='U',num_units=5,fplist=['SOIC*7.5x15.4mm*Pitch1.27mm*', 'TSSOP*4.4x7.8mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='2',name='A0',do_erc=True),
            Pin(num='6',name='V+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='A2',do_erc=True),
            Pin(num='12',name='~WP',do_erc=True),
            Pin(num='13',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='SCL',do_erc=True),
            Pin(num='14',name='A1',do_erc=True),
            Pin(num='24',name='A3',do_erc=True),
            Pin(num='18',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='V-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='RL0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='RH0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='RW0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='RL1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='RH1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='RW1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='RW2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='RH2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='22',name='RL2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='RW3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='RH3',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='RL3',func=Pin.PASSIVE,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

cmos_ieee = SchLib(tool=SKIDL).add_parts(*[
        Part(name='4001',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4002',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4006',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4008',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4009',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4010',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40104',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40106',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['4584']),
        Part(name='4011',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40110',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4012',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4013',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4014',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4015',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4016',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['4066']),
        Part(name='4017',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40174',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40175',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4018',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4019',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40192',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40193',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40194',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4020',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4021',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4022',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4023',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4024',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40240',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['40244']),
        Part(name='40245',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4025',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4027',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4028',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4029',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4030',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40373',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='40374',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4040',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4041',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4042',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4043',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4044',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4046',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4048',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4049',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4050',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4051',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4052',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4053',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4067',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4068',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4069',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4070',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4071',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4072',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4073',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4075',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4077',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4078',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4081',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4082',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4093',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4095',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4096',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4099',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4104',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4160',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['40160']),
        Part(name='4161',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['40161']),
        Part(name='4162',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['40162']),
        Part(name='4163',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['40163']),
        Part(name='4174',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4175',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4502',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4504',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4507',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4508',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4510',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4511',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4512',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4514',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4515',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4518',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4520',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4529',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4530',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4538',dest=TEMPLATE,tool=SKIDL,do_erc=True,aliases=['4528']),
        Part(name='4539',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4543',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4555',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4556',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='4585',dest=TEMPLATE,tool=SKIDL,do_erc=True)])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

wiznet = SchLib(tool=SKIDL).add_parts(*[
        Part(name='W5100',dest=TEMPLATE,tool=SKIDL,keywords='Wiznet Ethernet controller',description='WizNet W5100 10/100Mb Ethernet controller with TCP/IP stack',ref_prefix='U',num_units=1,fplist=['LQFP*10x10mm*Pitch0.4mm*'],do_erc=True,pins=[
            Pin(num='1',name='RSET_BG',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='VCC3V3A',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='GNDA',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='RXIP',do_erc=True),
            Pin(num='6',name='RXIN',do_erc=True),
            Pin(num='7',name='VCC1V8A',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='TXOP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='TXON',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='GNDA',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='DATA6',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='SCLK',do_erc=True),
            Pin(num='40',name='ADDR12',do_erc=True),
            Pin(num='50',name='ADDR4',do_erc=True),
            Pin(num='60',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='70',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='11',name='V18',func=Pin.PWROUT,do_erc=True),
            Pin(num='21',name='DATA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='SEN',do_erc=True),
            Pin(num='41',name='ADDR11',do_erc=True),
            Pin(num='51',name='ADDR3',do_erc=True),
            Pin(num='61',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='71',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='12',name='VCC3V3D',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='DATA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='42',name='ADDR10',do_erc=True),
            Pin(num='52',name='ADDR2',do_erc=True),
            Pin(num='62',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='72',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='13',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='DATA3',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='VCC1V8D',func=Pin.PWRIN,do_erc=True),
            Pin(num='43',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='53',name='ADDR1',do_erc=True),
            Pin(num='63',name='OPMODE0',do_erc=True),
            Pin(num='73',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='14',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='DATA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='TEST_MODE3',do_erc=True),
            Pin(num='44',name='VCC3V3D',func=Pin.PWRIN,do_erc=True),
            Pin(num='54',name='ADDR0',do_erc=True),
            Pin(num='64',name='OPMODE1',do_erc=True),
            Pin(num='74',name='VCC1V8A',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='VCC1V8D',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='DATA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='TEST_MODE2',do_erc=True),
            Pin(num='45',name='ADDR9',do_erc=True),
            Pin(num='55',name='CS',do_erc=True),
            Pin(num='65',name='OPMODE2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='75',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='16',name='VCC1V8D',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='DATA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='TEST_MODE1',do_erc=True),
            Pin(num='46',name='ADDR8',do_erc=True),
            Pin(num='56',name='INT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='66',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='76',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='17',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='MISO',do_erc=True),
            Pin(num='37',name='TEST_MODE0',do_erc=True),
            Pin(num='47',name='ADDR7',do_erc=True),
            Pin(num='57',name='WR',do_erc=True),
            Pin(num='67',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='77',name='GNDA',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='VCC3V3D',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='MOSI',do_erc=True),
            Pin(num='38',name='ADDR14',do_erc=True),
            Pin(num='48',name='ADDR6',do_erc=True),
            Pin(num='58',name='RD',do_erc=True),
            Pin(num='68',name='GNDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='DATA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='SCS',do_erc=True),
            Pin(num='39',name='ADDR13',do_erc=True),
            Pin(num='49',name='ADDR5',do_erc=True),
            Pin(num='59',name='RESET',do_erc=True),
            Pin(num='69',name='VCC1V8D',func=Pin.PWRIN,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

digital_audio = SchLib(tool=SKIDL).add_parts(*[
        Part(name='AK5392VS',dest=TEMPLATE,tool=SKIDL,keywords='24bit Sigma Delta Audio ADC 2ch',description='AK5392-VS, Enhanced Audio ADC, 2 channels Sigma Delta, 24bit, SO28',ref_prefix='U',num_units=1,fplist=['SO*'],do_erc=True,pins=[
            Pin(num='1',name='VREFL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GNDL',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCOML',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='AINL+',do_erc=True),
            Pin(num='5',name='AINL-',do_erc=True),
            Pin(num='6',name='ZCAL',do_erc=True),
            Pin(num='7',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='CAL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='~RST~',do_erc=True),
            Pin(num='20',name='TEST',do_erc=True),
            Pin(num='11',name='SMODE2',do_erc=True),
            Pin(num='21',name='BGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='SMODE1',do_erc=True),
            Pin(num='22',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='LRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='SCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='AINR-',do_erc=True),
            Pin(num='15',name='SDATA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='AINR+',do_erc=True),
            Pin(num='16',name='FSYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='VCOMR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='CLK',do_erc=True),
            Pin(num='27',name='GNDR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='CMODE',do_erc=True),
            Pin(num='28',name='VREFR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='HPFE',do_erc=True)]),
        Part(name='AK5393VS',dest=TEMPLATE,tool=SKIDL,keywords='96kHz 24bit Sigma Delta Audio ADC 2ch',description='Enhanced Audio ADC, 2 channels Sigma Delta, 24bit 96kHz, SO28',ref_prefix='U',num_units=1,fplist=['SO*'],do_erc=True,pins=[
            Pin(num='1',name='VREFL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='GNDL',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VCOML',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='AINL+',do_erc=True),
            Pin(num='5',name='AINL-',do_erc=True),
            Pin(num='6',name='ZCAL',do_erc=True),
            Pin(num='7',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='CAL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='~RST~',do_erc=True),
            Pin(num='20',name='TEST',do_erc=True),
            Pin(num='11',name='SMODE2',do_erc=True),
            Pin(num='21',name='BGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='SMODE1',do_erc=True),
            Pin(num='22',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='LRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='SCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='AINR-',do_erc=True),
            Pin(num='15',name='SDATA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='AINR+',do_erc=True),
            Pin(num='16',name='FSYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='VCOMR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='MCLK',do_erc=True),
            Pin(num='27',name='GNDR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='DFS',do_erc=True),
            Pin(num='28',name='VREFR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='HPFE',do_erc=True)]),
        Part(name='AK5394AVS',dest=TEMPLATE,tool=SKIDL,keywords='192kHz 24bit Sigma Delta Audio ADC 2ch',description='Super High Performance Audio ADC, 2 channels Sigma Delta, 24bit 192kHz, SO28',ref_prefix='U',num_units=1,fplist=['SO*'],do_erc=True,pins=[
            Pin(num='1',name='VREFL+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='VREFL-',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='VCOML',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='AINL+',do_erc=True),
            Pin(num='5',name='AINL-',do_erc=True),
            Pin(num='6',name='ZCAL',do_erc=True),
            Pin(num='7',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='CAL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='~RST~',do_erc=True),
            Pin(num='20',name='DFS1',do_erc=True),
            Pin(num='11',name='SMODE2',do_erc=True),
            Pin(num='21',name='BGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='SMODE1',do_erc=True),
            Pin(num='22',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='LRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='SCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='AINR-',do_erc=True),
            Pin(num='15',name='SDATA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='AINR+',do_erc=True),
            Pin(num='16',name='FSYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='VCOMR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='MCLK',do_erc=True),
            Pin(num='27',name='VREFR-',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='DFS0',do_erc=True),
            Pin(num='28',name='VREFR+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='HPFE',do_erc=True)]),
        Part(name='CS4245',dest=TEMPLATE,tool=SKIDL,keywords='CS4245 stereo audio codec',description='Stereo Audio CODEC, 104 dB, 24-Bit, 192 kHz',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='SDA/CDOUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='SCL/CCLK',do_erc=True),
            Pin(num='3',name='AD0/~CS',do_erc=True),
            Pin(num='4',name='AD1/CDIN',do_erc=True),
            Pin(num='5',name='VLC',do_erc=True),
            Pin(num='6',name='~RESET',do_erc=True),
            Pin(num='7',name='AIN3A',do_erc=True),
            Pin(num='8',name='AIN3B',do_erc=True),
            Pin(num='9',name='AIN2A',do_erc=True),
            Pin(num='10',name='AIN2B',do_erc=True),
            Pin(num='20',name='FILT2+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='30',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='40',name='MCLK2',do_erc=True),
            Pin(num='11',name='AIN1A',do_erc=True),
            Pin(num='21',name='AIN4A/MICIN1',do_erc=True),
            Pin(num='31',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='41',name='SDOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='AIN1B',do_erc=True),
            Pin(num='22',name='AIN4B/MICIN2',do_erc=True),
            Pin(num='32',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='42',name='SCLK1',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='AIN5A',do_erc=True),
            Pin(num='33',name='AOUTA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='43',name='LRCK1',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='AIN5B',do_erc=True),
            Pin(num='34',name='AOUTB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='44',name='MCLK1',do_erc=True),
            Pin(num='15',name='AFILTA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='MICBIAS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='35',name='~MUTEC',func=Pin.OUTPUT,do_erc=True),
            Pin(num='45',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='AFILTB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='26',name='AIN6A',do_erc=True),
            Pin(num='36',name='VLS',func=Pin.PWRIN,do_erc=True),
            Pin(num='46',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='VQ1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='27',name='AIN6B',do_erc=True),
            Pin(num='37',name='SDIN',do_erc=True),
            Pin(num='47',name='INT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='VQ2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='28',name='AUXOUTA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='38',name='SCLK2',func=Pin.BIDIR,do_erc=True),
            Pin(num='48',name='OVFL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='FILT1+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='29',name='AUXOUTB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='39',name='LRCK2',func=Pin.BIDIR,do_erc=True)]),
        Part(name='CS43L21',dest=TEMPLATE,tool=SKIDL,keywords='stereo audio dac',description='Stereo Audio DAC, 24-bit, 96 kHz, 98 dB',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='LRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='SDA/MCLKDIV2',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='SCL/CCLK/I2S/~LJ',do_erc=True),
            Pin(num='4',name='AD0/~CS~/DEM',do_erc=True),
            Pin(num='5',name='VA_HP',do_erc=True),
            Pin(num='6',name='FLYP',do_erc=True),
            Pin(num='7',name='GND_HP',do_erc=True),
            Pin(num='8',name='FLYN',do_erc=True),
            Pin(num='9',name='VSS_HP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='AOUTB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='30',name='MCLK',do_erc=True),
            Pin(num='11',name='AOUTA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='31',name='SCLK',do_erc=True),
            Pin(num='12',name='VA',do_erc=True),
            Pin(num='32',name='SDIN',do_erc=True),
            Pin(num='13',name='AGND',do_erc=True),
            Pin(num='14',name='FILT+',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='VQ',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='~RESET',do_erc=True),
            Pin(num='26',name='VL',do_erc=True),
            Pin(num='27',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='29',name='TESTO/M/~S',do_erc=True)]),
        Part(name='CS5361',dest=TEMPLATE,tool=SKIDL,keywords='stereo audio adc',description='Stereo Audio ADC, 24 bits, 192 kHz, 114 dB',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='RST',do_erc=True),
            Pin(num='2',name='M/~S',do_erc=True),
            Pin(num='3',name='LRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='SCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='MCLK',do_erc=True),
            Pin(num='6',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VL',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='SDOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='MDIV',do_erc=True),
            Pin(num='20',name='AINR-',do_erc=True),
            Pin(num='11',name='~HPF',do_erc=True),
            Pin(num='21',name='AINR+',do_erc=True),
            Pin(num='12',name='I2S/~LJ',do_erc=True),
            Pin(num='22',name='VQ',func=Pin.PWROUT,do_erc=True),
            Pin(num='13',name='M0',do_erc=True),
            Pin(num='23',name='REFGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='M1',do_erc=True),
            Pin(num='24',name='FILT+',func=Pin.PWROUT,do_erc=True),
            Pin(num='15',name='~OVFL',do_erc=True),
            Pin(num='16',name='AINL+',do_erc=True),
            Pin(num='17',name='AINL-',do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='VA',func=Pin.PWRIN,do_erc=True)]),
        Part(name='CS8406',dest=TEMPLATE,tool=SKIDL,keywords='digital audio interface transmitter spdif',description='192 kHz Digital Audio Interface Transmitter (SOIC-28)',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='6',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='~RST',do_erc=True),
            Pin(num='21',name='OMCK',do_erc=True),
            Pin(num='12',name='ILRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='ISCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='VL',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='SDIN',do_erc=True),
            Pin(num='24',name='H/~S',do_erc=True),
            Pin(num='25',name='TXN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='26',name='TXP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='1',name='COPY/C',do_erc=True),
            Pin(num='3',name='~EMPH',do_erc=True),
            Pin(num='4',name='SFMT0',do_erc=True),
            Pin(num='5',name='SFMT1',do_erc=True),
            Pin(num='10',name='APMS',do_erc=True),
            Pin(num='20',name='HWCK0',do_erc=True),
            Pin(num='11',name='TCBLD',do_erc=True),
            Pin(num='15',name='TCBL',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='CEN',do_erc=True),
            Pin(num='17',name='U',do_erc=True),
            Pin(num='27',name='HWCK1',do_erc=True),
            Pin(num='18',name='V',do_erc=True),
            Pin(num='28',name='ORIG',do_erc=True),
            Pin(num='19',name='~AUDIO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='1',name='SDA/CDOUT',do_erc=True),
            Pin(num='2',name='AD0/CS',do_erc=True),
            Pin(num='3',name='AD2',do_erc=True),
            Pin(num='4',name='RXP',do_erc=True),
            Pin(num='5',name='TEST',do_erc=True),
            Pin(num='10',name='TEST',do_erc=True),
            Pin(num='20',name='Bit_User',do_erc=True),
            Pin(num='11',name='TEST',do_erc=True),
            Pin(num='15',name='TCBL',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='TEST',do_erc=True),
            Pin(num='17',name='TEST',do_erc=True),
            Pin(num='27',name='AD1/CDIN',do_erc=True),
            Pin(num='18',name='TEST',do_erc=True),
            Pin(num='28',name='SCL/CCLK',do_erc=True),
            Pin(num='19',name='INT',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='CS8414',dest=TEMPLATE,tool=SKIDL,keywords='digital audio interface receiver spdif',description='96KHz Digital Audio Receiver',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='C',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='CD/F1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='CC/F0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='CB/E2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='CA/E1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='C0/E0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='RXP',do_erc=True),
            Pin(num='10',name='RXN',do_erc=True),
            Pin(num='20',name='FILT',do_erc=True),
            Pin(num='11',name='FSYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='SCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='CS12/FCK',do_erc=True),
            Pin(num='23',name='M0',do_erc=True),
            Pin(num='14',name='U',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='M1',do_erc=True),
            Pin(num='15',name='CBL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='ERF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='SEL',do_erc=True),
            Pin(num='26',name='SDATA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='M3',do_erc=True),
            Pin(num='27',name='CE/F2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='M2',do_erc=True),
            Pin(num='28',name='VERF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='MCK',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='CS8416-N',dest=TEMPLATE,tool=SKIDL,keywords='digital audio interface receiver spdif',description='192 kHz Digital Audio Interface Receiver (QFN-28)',ref_prefix='U',num_units=1,fplist=['QFN*28*'],do_erc=True,pins=[
            Pin(num='1',name='RXP0',do_erc=True),
            Pin(num='2',name='RXN',do_erc=True),
            Pin(num='3',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='FILT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~RESET',do_erc=True),
            Pin(num='7',name='RXP4/RXSEL1',do_erc=True),
            Pin(num='8',name='RXP5/RXSEL0',do_erc=True),
            Pin(num='9',name='RXP6/TXSEL1',do_erc=True),
            Pin(num='10',name='RXP7/TXSEL0',do_erc=True),
            Pin(num='20',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='AD0/~CS~/NV/RERR',do_erc=True),
            Pin(num='21',name='RMCK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='AD1/CDIN/~AUDIO',do_erc=True),
            Pin(num='22',name='OMCK',do_erc=True),
            Pin(num='13',name='SCL/CCLK/96KHZ',do_erc=True),
            Pin(num='23',name='SDOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='SDA/CDOUT/RCBL',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='OSCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='U/AD2/GPO2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='OLRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='C/GPO1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='26',name='RXP3',do_erc=True),
            Pin(num='17',name='TX/GPO0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='27',name='RXP2',do_erc=True),
            Pin(num='18',name='VL',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='RXP1',do_erc=True),
            Pin(num='19',name='DGND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='CS8416-Z',dest=TEMPLATE,tool=SKIDL,keywords='digital audio interface receiver spdif',description='192 kHz Digital Audio Interface Receiver (TSSOP-28)',ref_prefix='U',num_units=1,fplist=['SOIC*28*', '*SSOP*28*'],do_erc=True,aliases=['CS8416-S', 'CS8416'],pins=[
            Pin(num='1',name='RXP3',do_erc=True),
            Pin(num='2',name='RXP2',do_erc=True),
            Pin(num='3',name='RXP1',do_erc=True),
            Pin(num='4',name='RXP0',do_erc=True),
            Pin(num='5',name='RXN',do_erc=True),
            Pin(num='6',name='VA',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='FILT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='~RESET',do_erc=True),
            Pin(num='10',name='RXP4/RXSEL1',do_erc=True),
            Pin(num='20',name='TX/GPO0',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='RXP5/RXSEL0',do_erc=True),
            Pin(num='21',name='VL',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='RXP6/TXSEL1',do_erc=True),
            Pin(num='22',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='RXP7/TXSEL0',do_erc=True),
            Pin(num='23',name='VD',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='AD0/~CS~/NV/RERR',do_erc=True),
            Pin(num='24',name='RMCK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='AD1/CDIN/~AUDIO',do_erc=True),
            Pin(num='25',name='OMCK',do_erc=True),
            Pin(num='16',name='SCL/CCLK/96KHZ',do_erc=True),
            Pin(num='26',name='SDOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='SDA/CDOUT/RCBL',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='OSCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='U/AD2/GPO2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='28',name='OLRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='C/GPO1',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='CS8420_SOFT',dest=TEMPLATE,tool=SKIDL,keywords='digital audio sample rate converter transceiver',description='Digital Audio Sample Rate Converter and Transceiver',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='SDA/CDOUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='AD0/CS-',do_erc=True),
            Pin(num='3',name='EMPH-/AD2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='RXP',do_erc=True),
            Pin(num='5',name='RXN',do_erc=True),
            Pin(num='6',name='VA',do_erc=True),
            Pin(num='7',name='AGND',do_erc=True),
            Pin(num='8',name='FILT',do_erc=True),
            Pin(num='9',name='RST',do_erc=True),
            Pin(num='10',name='RMCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='U',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='RERR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='21',name='OMCK',do_erc=True),
            Pin(num='12',name='ILRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='DGND',do_erc=True),
            Pin(num='13',name='ISCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='VD',do_erc=True),
            Pin(num='14',name='SDIN',do_erc=True),
            Pin(num='24',name='H/S-',do_erc=True),
            Pin(num='15',name='TCBL',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='TXN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='OSCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='TXP',func=Pin.OUTPUT,do_erc=True),
            Pin(num='17',name='OLRCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='AD1/CDIN',do_erc=True),
            Pin(num='18',name='SDOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='28',name='SCL/CCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='INT',func=Pin.OPENCOLL,do_erc=True)]),
        Part(name='LM4811',dest=TEMPLATE,tool=SKIDL,keywords='headphone amplifier digital volume',description='Dual105mW Headphone Amplifier, Digital Volume Control, Shutdown Mode',ref_prefix='U',num_units=1,fplist=['VSSOP*', 'WSON*', 'SON*'],do_erc=True,pins=[
            Pin(num='1',name='VOUT1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='VIN1',do_erc=True),
            Pin(num='3',name='BYPASS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='CLOCK',do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='UP/DN',do_erc=True),
            Pin(num='7',name='SHDN',do_erc=True),
            Pin(num='8',name='VIN2',do_erc=True),
            Pin(num='9',name='VOUT2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TLV320AIC23BPW',dest=TEMPLATE,tool=SKIDL,keywords='Stero Audio CODEC 96kHz Headphone',description='8-96kHz Stero Audio CODEC w/ Headphone Amp, TSSOP28',ref_prefix='U',num_units=1,fplist=['TSSOP*'],do_erc=True,pins=[
            Pin(num='1',name='BVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='CLKOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='BCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='DIN',do_erc=True),
            Pin(num='5',name='LRCIN',do_erc=True),
            Pin(num='6',name='DOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='LRCOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='HPVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='LHPOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='RHPOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='LLINEIN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='HPGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='~CS~',do_erc=True),
            Pin(num='12',name='LOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='22',name='MODE',do_erc=True),
            Pin(num='13',name='ROUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='23',name='SDIN',do_erc=True),
            Pin(num='14',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='SCLK',do_erc=True),
            Pin(num='15',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='XTI/MCK',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='VMID',func=Pin.PASSIVE,do_erc=True),
            Pin(num='26',name='XTO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='MICBIAS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='27',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='MICIN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='28',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='RLINEIN',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='TLV320AIC23BRHD',dest=TEMPLATE,tool=SKIDL,keywords='Stero Audio CODEC 96kHz Headphone',description='8-96kHz Stero Audio CODEC w/ Headphone Amp, QFN28',ref_prefix='U',num_units=1,fplist=['QFN*'],do_erc=True,pins=[
            Pin(num='1',name='DIN',do_erc=True),
            Pin(num='2',name='LRCIN',do_erc=True),
            Pin(num='3',name='DOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='LRCOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='HPVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='LHPOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='RHPOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='8',name='HPGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='LOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='ROUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='SDIN',do_erc=True),
            Pin(num='11',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='SCLK',do_erc=True),
            Pin(num='12',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='XTI/MCK',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='VMID',func=Pin.PASSIVE,do_erc=True),
            Pin(num='23',name='XTO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='MICBIAS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='24',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='MICIN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='25',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='RLINEIN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='26',name='BVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='LLINEIN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='27',name='CLKOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='~CS~',do_erc=True),
            Pin(num='28',name='BCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='MODE',do_erc=True)]),
        Part(name='TPA5050',dest=TEMPLATE,tool=SKIDL,keywords='AUDIO',description='Stereo Digital Audio Delay Processor With I2C Control',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='LRCLK',do_erc=True),
            Pin(num='2',name='DATA',do_erc=True),
            Pin(num='3',name='SCL',do_erc=True),
            Pin(num='4',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='ADD0',do_erc=True),
            Pin(num='11',name='ADD1',do_erc=True),
            Pin(num='12',name='ADD2',do_erc=True),
            Pin(num='13',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='DATA_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='BCLK',do_erc=True)])])